# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: table.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='table.proto',
  package='pydb.proto',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0btable.proto\x12\npydb.proto\"\x16\n\x05\x44ummy\x12\r\n\x05\x64ummy\x18\x01 \x01(\t\"\x83\x01\n\x05Table\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\"\n\x06schema\x18\x02 \x01(\x0b\x32\x12.pydb.proto.Schema\x12,\n\x0csub_table_id\x18\x04 \x03(\x0b\x32\x16.pydb.proto.SubTableId\x12\x1a\n\rindex_density\x18\x05 \x01(\x03:\x03\x32\x35\x36\"v\n\x0cStringRefMap\x12\x36\n\x07mapping\x18\x01 \x03(\x0b\x32%.pydb.proto.StringRefMap.MappingEntry\x1a.\n\x0cMappingEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xaa\x01\n\x0c\x43olumnSchema\x12\x0c\n\x04name\x18\x01 \x01(\t\x12+\n\x04type\x18\x02 \x01(\x0e\x32\x1d.pydb.proto.ColumnSchema.Type\x12\x10\n\x05width\x18\x03 \x01(\r:\x01\x31\"M\n\x04Type\x12\t\n\x05\x46LOAT\x10\x00\x12\n\n\x06\x44OUBLE\x10\x01\x12\t\n\x05INT32\x10\x02\x12\t\n\x05INT64\x10\x03\x12\x08\n\x04\x42YTE\x10\x04\x12\x0e\n\nSTRING_REF\x10\x05\"d\n\x06Schema\x12\x16\n\x0btime_column\x18\x01 \x01(\t:\x01t\x12\x12\n\ntag_column\x18\x02 \x03(\t\x12.\n\x0cvalue_column\x18\x03 \x03(\x0b\x32\x18.pydb.proto.ColumnSchema\"\x83\x01\n\nSubTableId\x12\n\n\x02id\x18\x01 \x01(\t\x12,\n\x03tag\x18\x02 \x03(\x0b\x32\x1f.pydb.proto.SubTableId.TagEntry\x12\x0f\n\x07str_tag\x18\x03 \x03(\t\x1a*\n\x08TagEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\"P\n\x08SubTable\x12\"\n\x02id\x18\x01 \x01(\x0b\x32\x16.pydb.proto.SubTableId\x12 \n\x05index\x18\x02 \x01(\x0b\x32\x11.pydb.proto.Index\"F\n\x05Index\x12\r\n\x05value\x18\x01 \x03(\x03\x12\x0b\n\x03pos\x18\x02 \x03(\x03\x12\x0f\n\x07last_ts\x18\x03 \x01(\x03\x12\x10\n\x08num_rows\x18\x04 \x01(\x04\"s\n\x0cTimeSelector\x12\r\n\x05start\x18\x01 \x01(\x03\x12\x1b\n\rinclude_start\x18\x02 \x01(\x08:\x04true\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x03\x12\x1a\n\x0binclude_end\x18\x04 \x01(\x08:\x05\x66\x61lse\x12\x0e\n\x06last_n\x18\x05 \x01(\x04\"*\n\x0bTagSelector\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x03(\t\"T\n\x10SubTableSelector\x12-\n\x0ctag_selector\x18\x01 \x03(\x0b\x32\x17.pydb.proto.TagSelector\x12\x11\n\ttag_order\x18\x04 \x03(\t\"\x85\x01\n\x08Selector\x12\x38\n\x12sub_table_selector\x18\x01 \x01(\x0b\x32\x1c.pydb.proto.SubTableSelector\x12/\n\rtime_selector\x18\x02 \x01(\x0b\x32\x18.pydb.proto.TimeSelector\x12\x0e\n\x06\x63olumn\x18\x03 \x03(\t\"\x86\x02\n\x0bQueryResult\x12\x10\n\x08num_rows\x18\x01 \x01(\x03\x12/\n\rcolumn_schema\x18\x02 \x03(\x0b\x32\x18.pydb.proto.ColumnSchema\x12I\n\x12\x63olumn_binary_data\x18\x03 \x03(\x0b\x32-.pydb.proto.QueryResult.ColumnBinaryDataEntry\x12\x30\n\x0estring_ref_map\x18\x04 \x01(\x0b\x32\x18.pydb.proto.StringRefMap\x1a\x37\n\x15\x43olumnBinaryDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01'
)



_COLUMNSCHEMA_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='pydb.proto.ColumnSchema.Type',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FLOAT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DOUBLE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INT32', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INT64', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BYTE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRING_REF', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=399,
  serialized_end=476,
)
_sym_db.RegisterEnumDescriptor(_COLUMNSCHEMA_TYPE)


_DUMMY = _descriptor.Descriptor(
  name='Dummy',
  full_name='pydb.proto.Dummy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dummy', full_name='pydb.proto.Dummy.dummy', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27,
  serialized_end=49,
)


_TABLE = _descriptor.Descriptor(
  name='Table',
  full_name='pydb.proto.Table',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pydb.proto.Table.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schema', full_name='pydb.proto.Table.schema', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sub_table_id', full_name='pydb.proto.Table.sub_table_id', index=2,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='index_density', full_name='pydb.proto.Table.index_density', index=3,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=True, default_value=256,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=52,
  serialized_end=183,
)


_STRINGREFMAP_MAPPINGENTRY = _descriptor.Descriptor(
  name='MappingEntry',
  full_name='pydb.proto.StringRefMap.MappingEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='pydb.proto.StringRefMap.MappingEntry.key', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='pydb.proto.StringRefMap.MappingEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=257,
  serialized_end=303,
)

_STRINGREFMAP = _descriptor.Descriptor(
  name='StringRefMap',
  full_name='pydb.proto.StringRefMap',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mapping', full_name='pydb.proto.StringRefMap.mapping', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STRINGREFMAP_MAPPINGENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=185,
  serialized_end=303,
)


_COLUMNSCHEMA = _descriptor.Descriptor(
  name='ColumnSchema',
  full_name='pydb.proto.ColumnSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pydb.proto.ColumnSchema.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='pydb.proto.ColumnSchema.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='width', full_name='pydb.proto.ColumnSchema.width', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _COLUMNSCHEMA_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=306,
  serialized_end=476,
)


_SCHEMA = _descriptor.Descriptor(
  name='Schema',
  full_name='pydb.proto.Schema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='time_column', full_name='pydb.proto.Schema.time_column', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=b"t".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag_column', full_name='pydb.proto.Schema.tag_column', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value_column', full_name='pydb.proto.Schema.value_column', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=478,
  serialized_end=578,
)


_SUBTABLEID_TAGENTRY = _descriptor.Descriptor(
  name='TagEntry',
  full_name='pydb.proto.SubTableId.TagEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='pydb.proto.SubTableId.TagEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='pydb.proto.SubTableId.TagEntry.value', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=670,
  serialized_end=712,
)

_SUBTABLEID = _descriptor.Descriptor(
  name='SubTableId',
  full_name='pydb.proto.SubTableId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='pydb.proto.SubTableId.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag', full_name='pydb.proto.SubTableId.tag', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='str_tag', full_name='pydb.proto.SubTableId.str_tag', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SUBTABLEID_TAGENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=581,
  serialized_end=712,
)


_SUBTABLE = _descriptor.Descriptor(
  name='SubTable',
  full_name='pydb.proto.SubTable',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='pydb.proto.SubTable.id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='index', full_name='pydb.proto.SubTable.index', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=714,
  serialized_end=794,
)


_INDEX = _descriptor.Descriptor(
  name='Index',
  full_name='pydb.proto.Index',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='pydb.proto.Index.value', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pos', full_name='pydb.proto.Index.pos', index=1,
      number=2, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_ts', full_name='pydb.proto.Index.last_ts', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_rows', full_name='pydb.proto.Index.num_rows', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=796,
  serialized_end=866,
)


_TIMESELECTOR = _descriptor.Descriptor(
  name='TimeSelector',
  full_name='pydb.proto.TimeSelector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='start', full_name='pydb.proto.TimeSelector.start', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='include_start', full_name='pydb.proto.TimeSelector.include_start', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end', full_name='pydb.proto.TimeSelector.end', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='include_end', full_name='pydb.proto.TimeSelector.include_end', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_n', full_name='pydb.proto.TimeSelector.last_n', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=868,
  serialized_end=983,
)


_TAGSELECTOR = _descriptor.Descriptor(
  name='TagSelector',
  full_name='pydb.proto.TagSelector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pydb.proto.TagSelector.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='pydb.proto.TagSelector.value', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=985,
  serialized_end=1027,
)


_SUBTABLESELECTOR = _descriptor.Descriptor(
  name='SubTableSelector',
  full_name='pydb.proto.SubTableSelector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tag_selector', full_name='pydb.proto.SubTableSelector.tag_selector', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag_order', full_name='pydb.proto.SubTableSelector.tag_order', index=1,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1029,
  serialized_end=1113,
)


_SELECTOR = _descriptor.Descriptor(
  name='Selector',
  full_name='pydb.proto.Selector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sub_table_selector', full_name='pydb.proto.Selector.sub_table_selector', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_selector', full_name='pydb.proto.Selector.time_selector', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='column', full_name='pydb.proto.Selector.column', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1116,
  serialized_end=1249,
)


_QUERYRESULT_COLUMNBINARYDATAENTRY = _descriptor.Descriptor(
  name='ColumnBinaryDataEntry',
  full_name='pydb.proto.QueryResult.ColumnBinaryDataEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='pydb.proto.QueryResult.ColumnBinaryDataEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='pydb.proto.QueryResult.ColumnBinaryDataEntry.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1459,
  serialized_end=1514,
)

_QUERYRESULT = _descriptor.Descriptor(
  name='QueryResult',
  full_name='pydb.proto.QueryResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_rows', full_name='pydb.proto.QueryResult.num_rows', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='column_schema', full_name='pydb.proto.QueryResult.column_schema', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='column_binary_data', full_name='pydb.proto.QueryResult.column_binary_data', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_ref_map', full_name='pydb.proto.QueryResult.string_ref_map', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_QUERYRESULT_COLUMNBINARYDATAENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1252,
  serialized_end=1514,
)

_TABLE.fields_by_name['schema'].message_type = _SCHEMA
_TABLE.fields_by_name['sub_table_id'].message_type = _SUBTABLEID
_STRINGREFMAP_MAPPINGENTRY.containing_type = _STRINGREFMAP
_STRINGREFMAP.fields_by_name['mapping'].message_type = _STRINGREFMAP_MAPPINGENTRY
_COLUMNSCHEMA.fields_by_name['type'].enum_type = _COLUMNSCHEMA_TYPE
_COLUMNSCHEMA_TYPE.containing_type = _COLUMNSCHEMA
_SCHEMA.fields_by_name['value_column'].message_type = _COLUMNSCHEMA
_SUBTABLEID_TAGENTRY.containing_type = _SUBTABLEID
_SUBTABLEID.fields_by_name['tag'].message_type = _SUBTABLEID_TAGENTRY
_SUBTABLE.fields_by_name['id'].message_type = _SUBTABLEID
_SUBTABLE.fields_by_name['index'].message_type = _INDEX
_SUBTABLESELECTOR.fields_by_name['tag_selector'].message_type = _TAGSELECTOR
_SELECTOR.fields_by_name['sub_table_selector'].message_type = _SUBTABLESELECTOR
_SELECTOR.fields_by_name['time_selector'].message_type = _TIMESELECTOR
_QUERYRESULT_COLUMNBINARYDATAENTRY.containing_type = _QUERYRESULT
_QUERYRESULT.fields_by_name['column_schema'].message_type = _COLUMNSCHEMA
_QUERYRESULT.fields_by_name['column_binary_data'].message_type = _QUERYRESULT_COLUMNBINARYDATAENTRY
_QUERYRESULT.fields_by_name['string_ref_map'].message_type = _STRINGREFMAP
DESCRIPTOR.message_types_by_name['Dummy'] = _DUMMY
DESCRIPTOR.message_types_by_name['Table'] = _TABLE
DESCRIPTOR.message_types_by_name['StringRefMap'] = _STRINGREFMAP
DESCRIPTOR.message_types_by_name['ColumnSchema'] = _COLUMNSCHEMA
DESCRIPTOR.message_types_by_name['Schema'] = _SCHEMA
DESCRIPTOR.message_types_by_name['SubTableId'] = _SUBTABLEID
DESCRIPTOR.message_types_by_name['SubTable'] = _SUBTABLE
DESCRIPTOR.message_types_by_name['Index'] = _INDEX
DESCRIPTOR.message_types_by_name['TimeSelector'] = _TIMESELECTOR
DESCRIPTOR.message_types_by_name['TagSelector'] = _TAGSELECTOR
DESCRIPTOR.message_types_by_name['SubTableSelector'] = _SUBTABLESELECTOR
DESCRIPTOR.message_types_by_name['Selector'] = _SELECTOR
DESCRIPTOR.message_types_by_name['QueryResult'] = _QUERYRESULT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Dummy = _reflection.GeneratedProtocolMessageType('Dummy', (_message.Message,), {
  'DESCRIPTOR' : _DUMMY,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.Dummy)
  })
_sym_db.RegisterMessage(Dummy)

Table = _reflection.GeneratedProtocolMessageType('Table', (_message.Message,), {
  'DESCRIPTOR' : _TABLE,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.Table)
  })
_sym_db.RegisterMessage(Table)

StringRefMap = _reflection.GeneratedProtocolMessageType('StringRefMap', (_message.Message,), {

  'MappingEntry' : _reflection.GeneratedProtocolMessageType('MappingEntry', (_message.Message,), {
    'DESCRIPTOR' : _STRINGREFMAP_MAPPINGENTRY,
    '__module__' : 'table_pb2'
    # @@protoc_insertion_point(class_scope:pydb.proto.StringRefMap.MappingEntry)
    })
  ,
  'DESCRIPTOR' : _STRINGREFMAP,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.StringRefMap)
  })
_sym_db.RegisterMessage(StringRefMap)
_sym_db.RegisterMessage(StringRefMap.MappingEntry)

ColumnSchema = _reflection.GeneratedProtocolMessageType('ColumnSchema', (_message.Message,), {
  'DESCRIPTOR' : _COLUMNSCHEMA,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.ColumnSchema)
  })
_sym_db.RegisterMessage(ColumnSchema)

Schema = _reflection.GeneratedProtocolMessageType('Schema', (_message.Message,), {
  'DESCRIPTOR' : _SCHEMA,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.Schema)
  })
_sym_db.RegisterMessage(Schema)

SubTableId = _reflection.GeneratedProtocolMessageType('SubTableId', (_message.Message,), {

  'TagEntry' : _reflection.GeneratedProtocolMessageType('TagEntry', (_message.Message,), {
    'DESCRIPTOR' : _SUBTABLEID_TAGENTRY,
    '__module__' : 'table_pb2'
    # @@protoc_insertion_point(class_scope:pydb.proto.SubTableId.TagEntry)
    })
  ,
  'DESCRIPTOR' : _SUBTABLEID,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.SubTableId)
  })
_sym_db.RegisterMessage(SubTableId)
_sym_db.RegisterMessage(SubTableId.TagEntry)

SubTable = _reflection.GeneratedProtocolMessageType('SubTable', (_message.Message,), {
  'DESCRIPTOR' : _SUBTABLE,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.SubTable)
  })
_sym_db.RegisterMessage(SubTable)

Index = _reflection.GeneratedProtocolMessageType('Index', (_message.Message,), {
  'DESCRIPTOR' : _INDEX,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.Index)
  })
_sym_db.RegisterMessage(Index)

TimeSelector = _reflection.GeneratedProtocolMessageType('TimeSelector', (_message.Message,), {
  'DESCRIPTOR' : _TIMESELECTOR,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.TimeSelector)
  })
_sym_db.RegisterMessage(TimeSelector)

TagSelector = _reflection.GeneratedProtocolMessageType('TagSelector', (_message.Message,), {
  'DESCRIPTOR' : _TAGSELECTOR,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.TagSelector)
  })
_sym_db.RegisterMessage(TagSelector)

SubTableSelector = _reflection.GeneratedProtocolMessageType('SubTableSelector', (_message.Message,), {
  'DESCRIPTOR' : _SUBTABLESELECTOR,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.SubTableSelector)
  })
_sym_db.RegisterMessage(SubTableSelector)

Selector = _reflection.GeneratedProtocolMessageType('Selector', (_message.Message,), {
  'DESCRIPTOR' : _SELECTOR,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.Selector)
  })
_sym_db.RegisterMessage(Selector)

QueryResult = _reflection.GeneratedProtocolMessageType('QueryResult', (_message.Message,), {

  'ColumnBinaryDataEntry' : _reflection.GeneratedProtocolMessageType('ColumnBinaryDataEntry', (_message.Message,), {
    'DESCRIPTOR' : _QUERYRESULT_COLUMNBINARYDATAENTRY,
    '__module__' : 'table_pb2'
    # @@protoc_insertion_point(class_scope:pydb.proto.QueryResult.ColumnBinaryDataEntry)
    })
  ,
  'DESCRIPTOR' : _QUERYRESULT,
  '__module__' : 'table_pb2'
  # @@protoc_insertion_point(class_scope:pydb.proto.QueryResult)
  })
_sym_db.RegisterMessage(QueryResult)
_sym_db.RegisterMessage(QueryResult.ColumnBinaryDataEntry)


_STRINGREFMAP_MAPPINGENTRY._options = None
_SUBTABLEID_TAGENTRY._options = None
_QUERYRESULT_COLUMNBINARYDATAENTRY._options = None
# @@protoc_insertion_point(module_scope)
